/*
 * Copyright (C) Christof Musik, bitbone AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <qfont.h>
#include <qpainter.h>
#include <qpen.h>
#include <qregexp.h>
#include <qpixmap.h>
#include <qbrush.h>
#include <cmath>
#include <qdatetime.h>
#include <qtooltip.h>

#include <qdatetime.h>

#include "graph.h"
#include "config.h"

/*!
 *   This class displays the graphical content.
 */
Graph::Graph( QWidget *parent, const char *name, WFlags flags ) : QWidget( parent, name, flags|WNoAutoErase ) {
    snmpvalues = new QValueVector<int>();
    //snmpvalues[0] = 0;
    snmpvalues->push_back(0);
    old_counter_value = -1;
    timeout = TIMEOUT;
    showLines = SHOW_LINES;
    showValue = false;
    showTimeline = SHOW_TIMELINE;
    type = GRAPHTYPE;
    unitfactor = 1;
    fixedMaximum = false;
    toplabel = SHOW_TOPLABEL;

    QToolTip::setWakeUpDelay(3000);

    valueTimer = new QTimer();
    connect (valueTimer, SIGNAL(timeout()), this, SLOT(hideInlineValue()));

    background = QColor(BACKGROUND_COLOR);
    foreground = QColor(FOREGROUND_COLOR);
    graphcolor = QColor(GRAPH_COLOR);
}

Graph::~Graph() {
}

/*! 
 * Calculates all points from values in snmpvalues
 */
QPointArray Graph::getPoints() {
    QPointArray pointarray(snmpvalues->count()+3);
    QValueVector<int>::iterator it;

    int i=0;
    int w = width();
    int h = height();
    float scale = 1l;

    pointarray.setPoint(0, w, h);
    scale = (float) h / (float) max_value;
    for( it = snmpvalues->end()-1; it != snmpvalues->begin()-1; it-- ) {
	int a = (int) ( (float) *it * scale );
	pointarray.setPoint(i+1, w-i, h-a);

	i++;
    }
    pointarray.setPoint(i+1, w-i, h);
    pointarray.setPoint(++i+1, 0, h);
    return pointarray;
}

/*!
 * Add a value in to snmpvalues
 * \param str String returned from SNMP-Agent
 */
bool Graph::put( QString str ) {
    QRegExp exp("(.*)= (.*): (.*)");
    if ( !exp.exactMatch(str) ) {
	svalues = str;
	old_type = "STRING";
	update();
    } else {
	QString value_str, type, oid;
	double value_int = 0.0;

	QStringList list = QStringList::split("+", str);
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
	    if (exp.exactMatch(*it)) {
		// check if type is not the same
		if (type != "" && type != exp.cap(2)) {
		    std::cerr << type << " and " << exp.cap(2) << " does not match" << std::endl;
		    cleanUp();
		    return false;
		} else {
		    oid = exp.cap(1);
		    type = exp.cap(2);
		    if (type == "Counter32" || type == "INTEGER" || type == "Gauge32")
			value_int += exp.cap(3).toDouble();
		    else {
			value_str += exp.cap(3);
			type = "STRING";
		    }
		}
	    }
	}

	// check if oid has changed
	if (named_oid != old_oid && old_oid != "") {
	    cleanUp();
	}
	old_oid = named_oid;
	old_type = type;

	if (type == "Counter32") {
	    if (old_counter_value >= 0) {
		int i = (int) (old_counter_value - value_int) / timeout;
		old_counter_value = value_int;

		if (i < 0) i = -1 * i;
		if (i > max_value && !fixedMaximum) max_value = i;

		snmpvalues->push_back(i);
	    }
	    else 
		old_counter_value = value_int;

	} else if (type == "INTEGER" || type == "Gauge32") {
	    if ((int) value_int >= 0) {
		int v = (int) value_int;
		if ( v > max_value && !fixedMaximum ) max_value = v;
		snmpvalues->push_back(v);
	    }
	} else if (type == "STRING") {
	    svalues = value_str;
	}
	
	if ( (snmpvalues->count() > (unsigned int) width()+1) && (!snmpvalues->empty()) ) {
	    if (snmpvalues->at(0) == max_value && !fixedMaximum) 
		checkNewMaximum(); //if the value that will be deleted is the maximum, check for a new one
	    snmpvalues->erase(snmpvalues->begin());
	}
	update();
    }
    return true;
}

bool Graph::put(QString str, QString oid) {
    setNamedOID(oid);
    return put(str);
}

void Graph::cleanUp() {
    old_counter_value = -1;
    delete snmpvalues;
    snmpvalues = new QValueVector<int>();
    //snmpvalues[0] = 0;
    snmpvalues->push_back(0);
    if (!fixedMaximum) max_value = 0;
    update();
}

/*!
 * Sets the timeout value
 * \param t value timeout
 */
void Graph::setTimeout(int t) {
    timeout = t;
    update();
}

/*!
 * Sets the current connected server to be displayed in this widget
 * \param str name of the server
 */
void Graph::setConnectedServer(QString str) {
    connected_server = str;
}

void Graph::paintEvent(QPaintEvent*) {
    QPixmap pixmap(width(), height());
    pixmap.setOptimization(QPixmap::MemoryOptim);
    pixmap.fill(background);
    QPainter paint(&pixmap);
    paint.setFont(font);

    if (old_type == "Counter32" || old_type == "INTEGER" || old_type == "Gauge32") {
	if (type == Graph::Lines) {
	    QValueVector<int>::iterator it;

	    int count = 0;
	    int w = width();
	    int h = height();
	    float scale = (float) h / (float) max_value;

	    // draws the graph
	    paint.setPen(graphcolor);
	    for( it = snmpvalues->end()-1; it != snmpvalues->begin()-1; it-- ) {
		int a = (int) ((float)*it * scale);
		paint.drawLine(w-count, h, w-count, h-a);

		count++;
	    }
	} else if (type == Graph::Polygon || type == Graph::Outline) {
	    paint.setPen(graphcolor);
	    QPointArray a = getPoints();
	    if (type != Graph::Outline) paint.setBrush( QBrush(graphcolor) );
	    paint.drawPolygon(a);
	}
	
	// draw the lines
	paint.setPen( QPen(foreground, 0, Qt::DotLine) );
	if ( unittype >= Graph::Bytes && unittype <= Graph::MBytes ) checkDataUnitFactor();
	if ( max_value > 4 ) {
	    QString str;
	    for (float i = 0.25; i < 1; i = i+0.25) {
		//draws the number in front of the line
		paint.setPen(foreground);
		switch ( unittype ) {
		    case Graph::Bytes:
		    case Graph::KBytes:
		    case Graph::MBytes:
		    case Graph::GBytes:
			{
			    float print_value = ( max_value*(1-i) ) / unitfactor;
			    int prec = 3;
			    if ( print_value >= 1000 && print_value < 1024 ) prec = 4;
			    str = QString::number( print_value, 'g', prec )+unitstring;
			}
			break;

		    case Graph::Percent:
			str = QString::number( (int) (max_value*(1-i)) )+"%";
			break;

		    default:
			if (max_value < 100)
			    str = QString::number( (float) max_value*(1-i) );
			else
			    str = QString::number( (int) (max_value*(1-i)) );

		}
		QRect str_size = paint.fontMetrics().boundingRect(str);
		paint.drawText(2, (int) ( height()*i )+( str_size.height()/2 ), str);
		//draw the line
		paint.setPen( QPen(foreground.dark(),0, Qt::SolidLine) );
		if (showLines) paint.drawLine(str_size.width()+5,(int) (height()*i),width(),(int) (height()*i));
	    }
	}

	// draw the headline
	paint.setPen(foreground);
	if ( max_value > 0 ) {
	    QString num;

	    switch ( unittype ) {
		case Graph::Bytes:
		case Graph::KBytes:
		case Graph::MBytes:
		case Graph::GBytes:
		    {
			float print_value = (float) max_value/unitfactor;
			int prec = 3;
			if ( print_value >= 1000 && print_value < 1024 ) prec = 4;
			num = QString::number( print_value, 'g', prec )+unitstring;
		    }
		    break;

		case Graph::Percent:
		    num = QString::number(max_value)+"%";
		    break;

		default:
		    num = QString::number(max_value);
	    }

	    QRect num_size = paint.fontMetrics().boundingRect(num);
	    paint.drawText( 2, num_size.height(), num );

	    if (toplabel) {
		QString srv = connected_server+" - "+named_oid;
		QRect srv_size = paint.fontMetrics().boundingRect(srv);

		//paint.drawText(2, srv_size.height(), num);
		paint.drawText( width()-srv_size.width()-2, srv_size.height(), srv );

		setMinimumWidth( srv_size.width() + num_size.width() + 10 );
		setMinimumHeight( 5 * num_size.height() + ( height() / 5 ) );
	    }
	} else {
	    if (toplabel) {
		QString srv = connected_server+" - "+named_oid;
		QRect srv_size = paint.fontMetrics().boundingRect( srv );

		paint.drawText( width()-srv_size.width()-2, srv_size.height(), srv );

		setMinimumWidth( srv_size.width()+10 );
	    }
	}

	// draw current value on middle-click
	if ( showValue ) {
	    // floating method
	    QString v = QString::number( clickValue );
	    QRect v_size = paint.fontMetrics().boundingRect(v);

	    // check if clickPos is in widget size
	    if ( clickPos.x() + v_size.width() > width() )
		clickPos.setX( width() - v_size.width() );

	    if ( clickPos.y() < v_size.height() )
		clickPos.setY( v_size.height() );

	    if ( clickPos.y() > height() )
		clickPos.setY( height() );

	    if ( clickPos.x() < 0 )
		clickPos.setX( 0 );

	    paint.drawText( clickPos, v );
	    // fixed method
	    // paint.drawText(width() - v_size.width() - 2, height()-2, v);
	}

	if ( showTimeline ) {
	    paint.setPen(foreground.dark());
	    QTime time;
	    time.start();
	    for ( int i = width(), min = 0; i > 0; i -= (60), min++ ) {
		if (min%3 == 0) {
		    paint.drawLine( i, height(), i, height()-10 );
		    QTime t = time.addSecs( -1*min*60*timeout );

		    QString d = QString::number(t.hour())+":";
		    t.minute() < 10 ? d += "0"+QString::number(t.minute()) : d += QString::number(t.minute());
		    QRect d_size = paint.fontMetrics().boundingRect( d );

		    paint.drawText( i+2, height(), d );
		}
		else
		    paint.drawLine(i, height(), i, height()-5 );
	    }
	}
    } else if (old_type == "STRING") {
	paint.setPen(graphcolor);
	QRect str_size = paint.fontMetrics().boundingRect(svalues);
	paint.drawText( 10,str_size.height(), svalues );
	setMinimumWidth( str_size.width()+20 );
	setMinimumHeight( str_size.height() );
    }

    bitBlt(this, 0, 0 , &pixmap);
    paint.end();
    pixmap.detach();
}

void Graph::mouseReleaseEvent(QMouseEvent *evt) {
    if (evt->button() == Qt::MidButton) {
	clickValue = getValue(evt->y());
	clickPos = evt->pos();
	emit clicked( clickValue );
    }
}

void Graph::resizeEvent(QResizeEvent *) {
    unsigned int w = width();
    if ( (snmpvalues->count() >= w) && (!snmpvalues->empty()) ) {
	snmpvalues->erase(snmpvalues->begin(), snmpvalues->begin()+(snmpvalues->count()-width()));
    }
    if (!fixedMaximum)checkNewMaximum();
}

/*!
 * Calculates the value to the given y-coordinate
 * \param y y-coordinate
 */
int Graph::getValue(int y) {
    int a = (int) ( (float) max_value * ( 1 - ( (float) y / (float) height() ) ) );

    if ( a < 0 ) a = 0;
    if ( a > max_value ) a = max_value;
    return a;
}

/*!
 * Sets the color of the background
 * \param bg new background color
 */
void Graph::setBackgroundColor ( QColor bg ) {
    if ( bg.isValid() ) {
	background = bg;
	update();
    }
}

/*!
 * \return color of the background
 */
QColor Graph::getBackgroundColor() {
    return background;
}

/*!
 * Sets the color of the foreground
 * \param fg new foreground color
 */
void Graph::setForegroundColor ( QColor fg ) {
    if ( fg.isValid() ) {
	foreground = fg;
	update();
    }
}

/*!
 * \return color of the foreground
 */
QColor Graph::getForegroundColor() {
    return foreground;
}

/*!
 * Sets the color of the graph
 * \param g new graph color
 */
void Graph::setGraphColor(QColor g) {
    if (g.isValid()) {
	graphcolor = g;
	update();
    }
}

/*!
 * \return color of the graph
 */
QColor Graph::getGraphColor() {
    return graphcolor;
}

/*!
 * Removes the current maximum.
 * Usefull to remove unrealistic values.
 */
void Graph::removeMax() {
    QValueVector<int>::iterator it;
    for( it = snmpvalues->begin(); it != snmpvalues->end(); it++ ) {
	if (*it == max_value) {
	    snmpvalues->erase(it);
	    if (!fixedMaximum) checkNewMaximum();
	    update();
	    return;
	}
    }
}

/*!
 * Iterates over all values and sets the new max_value
 */
void Graph::checkNewMaximum() {
    QValueVector<int>::iterator it;
    int lmax = 0;
    if ( !snmpvalues->isEmpty()) {
	for( it = snmpvalues->begin()+1; it != snmpvalues->end(); ++it ) {
	    if (*it > lmax) lmax = *it;
	}
	max_value = lmax;
    }
}

/*!
 * Sets the name of the current OID, that is displayed
 * \param s new oidname
 */
void Graph::setNamedOID(QString s) {
    named_oid = s;
    QToolTip::add( this, connected_server+" - "+named_oid );
}

/*!
 * \param b if true lines will be shown
 */
void Graph::setShowLines(bool b) {
    showLines = b;
    update();
}

/*!
 * \return if true lines are shown
 */
bool Graph::getShowLines() {
    return showLines;
}

/**
 * Changes the font of this widget
 * \return f new font
 */
void Graph::setGraphFont(QFont f) {
	font = f;
	update();
}

/*!
 * \return font of this widget
 */
QFont Graph::getGraphFont() {
    return font;
}

/*!
 * Sets the type of the graph.
 * \param t Type
 * \see Type
 */
void Graph::setType(int t) {
    if (type > 0 && type <= Graph::TypeCounter) type = t;
}

/*!
 * \return type of the graph
 */
int Graph::getType() {
    return type;
}

void Graph::switchType() {
    type++;
    if (type > Graph::TypeCounter) type = 1;
    update();
}

void Graph::showInlineValue() {
    valueTimer->start(OSD_DISPLAY_TIMEOUT*1000, true);	
    showValue = true;
    update();
}

void Graph::hideInlineValue() {
    showValue = false;
    update();
}

/*!
 * Sets the property to show the timeline
 * \param b if true timeline will be shown
 */
void Graph::setShowTimeline(bool b) {
    showTimeline = b;
    update();
}

/*!
 * \return true if timeline is shown
 */
bool Graph::getShowTimeline() {
    return showTimeline;
}

void Graph::checkDataUnitFactor() {
    int max = max_value;

    // convert current maximum in bytes, according to the given unit
    switch (unittype) {
	case Graph::Bytes:
	    break;
	case Graph::KBytes:
	    max = max*1024;
	    break;
	case Graph::MBytes:
	    max = max*(int)pow(1024,2);
	    break;
	case Graph::GBytes:
	    max = max*(int)pow(1024,3);
	    break;
    }

    if ( max < 1024 ) {
	unitfactor = 1;
	unitstring = "";
    }
    else if ( max >= 1024 && max < (int) pow(1024,2) ) {
	unitfactor = (int) pow(1024,1-unittype);
	unitstring = "K";
    }
    else if ( max >= (int) pow(1024,2) && max < (int) pow(1024,3) ) {
	unitfactor = (int) pow(1024, 2-unittype);
	unitstring = "M";
    }
    else if ( max >= (int) pow(1024,3) && max < (int) pow(1024,4) ) {
	unitfactor = (int) pow(1024, 3-unittype);
	unitstring = "G";
    }
    else if ( max >= (int) pow(1024,4) && max < (int) pow(1024,5) ) {
	unitfactor = (int) pow(1024, 4-unittype);
	unitstring = "P";
    }
}

/*!
 * Sets the unit of the current graph
 * \param type Unit value
 * \see Unit
 */
void Graph::setUnitType(int type) {
    unittype = type;
}

void Graph::setFixedMaximum(int max) {
    if (max < 0) {
	fixedMaximum = false;
	checkNewMaximum();
    } else {
	fixedMaximum = true;
	max_value = max;
    }
}

void Graph::setShowToplabel (bool b) {
    toplabel = b;
    setMinimumWidth(0);
}

bool Graph::getShowToplabel () {
    return toplabel;
}
