/*
 * Copyright (C) Christof Musik, bitbone AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <qcolordialog.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qfontdialog.h>
#include <qlistview.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qstring.h>

#include "monitoritem.h"
#include "monitorpreviewdlg.h"
#include "config.h"

/*!
 * Reads a Monitor file and displays all entrys in the dialog
 * \param f filename
 */
void monitorfiledlg::readMonFile( QString f ) {
    if (f == QString::null || f == "") {
	return;
    }

    QFile file(f);
    if (file.open(IO_ReadOnly)) {
	lvMonitors->clear();
	QTextStream stream(&file);
	QString line;

	QRegExp configend ("^\\s*\\[end\\].*");
	setCaption(tr("Monitor File Editor - %1").arg(f));
	
	SNMPMonConfig *cfg = new SNMPMonConfig();

	int ln = 1;

	while(!stream.atEnd()) {
	    line = stream.readLine();
	    if (configend.exactMatch(line)) {
		if (cfg->server != "" && cfg->oidname != "") {
		    new MonitorItem(lvMonitors, lvMonitors->lastItem(), cfg->server, cfg->oidname, cfg);
		} else {
		    std::cerr << tr("Server or oidname not defined") << std::endl;
		}
		cfg = new SNMPMonConfig();	
	    } else {
		if (!cfg->addConfigString(line)) {
		    std::cerr << tr("Error on line %1").arg(ln) << std::endl;
		}
	    }
	    ln++;
	}

	lvMonitors->setSelected(lvMonitors->firstChild(), true);
	changed = false;
	filechanged = false;
	delete cfg;
    } else {
	std::cerr << tr("Couldn't open file %1").arg(f) << std::endl;
    }
}

/*!
 * Saves the current setting into a file
 * \param filename filename
 */
void monitorfiledlg::saveMonFile(QString filename) {
    QListViewItemIterator it (lvMonitors);
    QFile file(filename);
    if (file.open(IO_WriteOnly)) {
	QTextStream stream(&file);
	QString error;
	while (it.current()) {
	    SNMPMonConfig *cfg = ((MonitorItem*)it.current())->config;
	    QString str = cfg->getSaveString();
	    if (str != "") 
		stream << str;
	    else
		error += " - "+it.current()->text(0)+"\n";
	    it++;
	}

	if (error != "") {
	    QMessageBox::warning(this, tr("Warning"), tr("Error in\n%1").arg(error), QMessageBox::Ok, QMessageBox::NoButton);
	}
    }
    filechanged = false;
}

void monitorfiledlg::init() {
    leFont->setText(QFont().family()+" "+QString::number(QFont().pointSize()));
    changed = false;
    filechanged = false;
    enableAll(false);
}

/*!
 * This method called every time a new item in lvMonitors is choosed
 */
void monitorfiledlg::changeMonitor() {
    if (changed) {
	int i = QMessageBox::warning(this, tr("Warning"), tr("Changes will be lost\nWould you like to save?"), QMessageBox::Yes, QMessageBox::No);
	if (i == QMessageBox::Yes) {
	    applyChanges();
	}
    }
    if (lvMonitors->selectedItem() != 0) {
	enableAll(true);
	cbAllbg->setChecked(false);
	cbAllfg->setChecked(false);
	cbAllgraph->setChecked(false);
	cbAllsize->setChecked(false);
	cbAllfont->setChecked(false);
	cbAllbars->setChecked(false);
	cbAlllines->setChecked(false);
	cbAlltype->setChecked(false);

	MonitorItem *item = (MonitorItem*)lvMonitors->selectedItem();
	SNMPMonConfig *cfg = item->config; 
	leServer->setText(cfg->server);
	leCommunity->setText(cfg->community);
	if (cfg->oidname != "")
	    cbOidname->setCurrentText(cfg->oidname);
	else
	    cbOidname->setCurrentItem(0);
	sbTimeout->setValue(cfg->timeout);
	sbVersion->setValue(cfg->version);
	cbToolbar->setChecked(cfg->toolbar);
	cbStatusbar->setChecked(cfg->statusbar);
	cbMenubar->setChecked(cfg->menubar);
	cbTitlebar->setChecked(cfg->titlebar);
	cbAutostart->setChecked(cfg->autostart);
	cbDockmode->setChecked(cfg->withdrawn);
	cbShowTlabel->setChecked(cfg->toplabel);
	sbX->setValue(cfg->x);
	sbY->setValue(cfg->y);
	sbH->setValue(cfg->h);
	sbW->setValue(cfg->w);
	pbBG->setPaletteBackgroundColor(cfg->bg);
	pbFG->setPaletteBackgroundColor(cfg->fg);
	pbGraph->setPaletteBackgroundColor(cfg->graph);
	font = cfg->font;
	leFont->setText(font.family()+" "+QString::number(font.pointSize()));
	cbShowlines->setChecked(cfg->lines);
	cbShowTline->setChecked(cfg->timeline);
	gbGraphtype->setButton(cfg->type);
	changed = false;
	toggleApplybtn();
	olditem = item;
    } else {
	clearAll();
	enableAll(false);
    }
}

/*!
 * Applys the all changes to memory
 */
void monitorfiledlg::applyChanges() {
    if (leServer->text() != "" && cbOidname->currentText() != "") {
	if (cbAllsize->isChecked() || cbAllbg->isChecked() || cbAllfg->isChecked() || cbAllgraph->isChecked() || cbAllfont->isChecked() || cbAllbars->isChecked() || cbAlllines->isChecked() || cbAlltype->isChecked()) {
	    QListViewItemIterator it (lvMonitors);
	    while (it.current()) {
		MonitorItem *item = (MonitorItem*)it.current();
		SNMPMonConfig *cfg = item->config;
		if (cbAllsize->isChecked()) {
		    cfg->w = sbW->value();
		    cfg->h = sbH->value();
		    cfg->withdrawn = cbDockmode->isChecked();
		}
		if (cbAllbg->isChecked()) 
		    cfg->bg = pbBG->paletteBackgroundColor();
		if (cbAllfg->isChecked())
		    cfg->fg = pbFG->paletteBackgroundColor();
		if (cbAllgraph->isChecked())
		    cfg->graph = pbGraph->paletteBackgroundColor();
		if (cbAllfont->isChecked())
		    cfg->font = font;
		if (cbAllbars->isChecked()) {
		    cfg->statusbar = cbStatusbar->isChecked();
		    cfg->titlebar = cbTitlebar->isChecked();
		    cfg->menubar = cbMenubar->isChecked();
		    cfg->toolbar = cbToolbar->isChecked();
		}
		if (cbAlllines->isChecked()) {
		    cfg->timeline = cbShowTline->isChecked();
		    cfg->lines = cbShowlines->isChecked();
		    cfg->toplabel = cbShowTlabel->isChecked();
		}
		if (cbAlltype->isChecked()) 
		    cfg->type = gbGraphtype->selectedId();
		changed = false;
		filechanged = true;
		toggleApplybtn();
		it++;
	    }
	} 

	SNMPMonConfig *cfg = olditem->config;
	cfg->server = leServer->text();
	cfg->oidname = cbOidname->currentText();
	cfg->community = leCommunity->text();
	cfg->toolbar = cbToolbar->isChecked();
	cfg->menubar = cbMenubar->isChecked();
	cfg->statusbar = cbStatusbar->isChecked();
	cfg->titlebar = cbTitlebar->isChecked();
	cfg->autostart = cbAutostart->isChecked();
	cfg->timeout = sbTimeout->value();
	cfg->version = sbVersion->value();
	cfg->x = sbX->value();
	cfg->y = sbY->value();
	cfg->w = sbW->value();
	cfg->h = sbH->value();
	cfg->bg = pbBG->paletteBackgroundColor();
	cfg->fg = pbFG->paletteBackgroundColor();
	cfg->graph = pbGraph->paletteBackgroundColor();
	cfg->font = font;
	cfg->lines = cbShowlines->isChecked();
	cfg->type = gbGraphtype->selectedId();
	cfg->timeline = cbShowTline->isChecked();
	cfg->lines = cbShowlines->isChecked();
	cfg->withdrawn = cbDockmode->isChecked();
	cfg->toplabel = cbShowTlabel->isChecked();
	olditem->setText(0, cfg->server);
	olditem->setText(1, cfg->oidname);

	cbAllbg->setChecked(false);
	cbAllfg->setChecked(false);
	cbAllgraph->setChecked(false);
	cbAllsize->setChecked(false);
	cbAllfont->setChecked(false);
	cbAllbars->setChecked(false);
	cbAlllines->setChecked(false);
	cbAlltype->setChecked(false);

	changed = false;
	filechanged = true;
	toggleApplybtn();
    }
}

/*!
 * First asks to save a changed file, and then opens a file-dialog
 * \see readMonFile
 */
void monitorfiledlg::openFile() {
    if (filechanged) {
	int i = QMessageBox::warning(this, tr("Warning"), tr("File has changed\nWould you like to save?"), QMessageBox::Yes, QMessageBox::No);
	if (i == QMessageBox::Yes) {
	    saveFile();
	    QString str = QFileDialog::getOpenFileName(QDir::currentDirPath(), tr("Monitor Files (*.mon);; All Files (*)"));
	    readMonFile(str);
	}
    }
    QString str = QFileDialog::getOpenFileName(QDir::currentDirPath(), tr("Monitor Files (*.mon);; All Files (*)"));
    readMonFile(str);
}

/*
 * If no color is set in config, the default colors will be used
 */
void monitorfiledlg::setDefaultColors( QColor b, QColor f, QColor g ) {
    defBGColor = b;
    defFGColor = f;
    defGraphColor = g;
}

/*!
 * Clears all input field and sets default values
 */
void monitorfiledlg::clearAll() {
    leServer->setText("");
    leCommunity->setText(COMMUNITY);
    cbOidname->setCurrentItem(0);
    sbTimeout->setValue(TIMEOUT);
    sbVersion->setValue(PROTOCOL_VERSION);
    cbToolbar->setChecked(SHOW_TOOLBAR);
    cbStatusbar->setChecked(SHOW_STATUSBAR);
    cbMenubar->setChecked(SHOW_MENUBAR);
    cbTitlebar->setChecked(SHOW_TITLEBAR);
    cbAutostart->setChecked(AUTOSTART);
    cbDockmode->setChecked(WITHDRAWN_STATE);
    cbShowTlabel->setChecked(SHOW_TOPLABEL);
    sbX->setValue(X_POS);
    sbY->setValue(Y_POS);
    sbH->setValue(HEIGHT);
    sbW->setValue(WIDTH);
    pbBG->setPaletteBackgroundColor(BACKGROUND_COLOR);
    pbFG->setPaletteBackgroundColor(FOREGROUND_COLOR);
    pbGraph->setPaletteBackgroundColor(GRAPH_COLOR);
    changed = false;
    toggleApplybtn();
}

/*!
 * \param e if true all views are enabled
 */
void monitorfiledlg::enableAll(bool e) {
    gbConnection->setEnabled(e);
    gbFonts->setEnabled(e);
    gbColors->setEnabled(e);
    gbSize->setEnabled(e);
    gbGraphtype->setEnabled(e);

    pbdelete->setEnabled(e);
}

/*!
 * Deletes the current selected entry
 */
void monitorfiledlg::deleteEntry() {
    MonitorItem *item = (MonitorItem*)lvMonitors->selectedItem();
    MonitorItem *n_item;
    if (item->itemAbove() == NULL) 
	n_item = (MonitorItem*)item->itemBelow();
    else
	n_item = (MonitorItem*)item->itemAbove();
    lvMonitors->setSelected(n_item, true);
    lvMonitors->ensureItemVisible(n_item);
    lvMonitors->takeItem(item);
    filechanged = true;
}

/*!
 * Adds a new entry with default values
 */
void monitorfiledlg::newEntry() {
    SNMPMonConfig *cfg = new SNMPMonConfig();
    MonitorItem *item = new MonitorItem(lvMonitors, lvMonitors->lastItem(), tr("new Item"), "", cfg);
    lvMonitors->setSelected(item, true);
    lvMonitors->ensureItemVisible(item);
    filechanged = true;
}

/*!
 * Asks for a filename and saves all entrys to this file
 * \see saveMonFile
 */
void monitorfiledlg::saveFile() {
    QString str = QFileDialog::getSaveFileName(QDir::currentDirPath(), tr("Monitor Files (*.mon);; All Files (*)"));
    if (str != "") {
	setCaption(tr("Monitor File Editor - %1").arg(str));
	saveMonFile(str);
    }
}

/*!
 * This method is called every time something is changed
 */
void monitorfiledlg::setChanged() {
    if (leServer->text() != "" && cbOidname->currentText() != "") {
	changed = true;
	if (changed) toggleApplybtn();
    }
}

void monitorfiledlg::toggleApplybtn() {
    if (changed && leServer->text() != "" && cbOidname->currentText() != "") {
	pbapply->setEnabled(true);
	pbReset->setEnabled(true);
    } else {
	pbapply->setEnabled(false);
	pbReset->setEnabled(false);
    }
}
/*!
 * Opens a QColorDialog and change the backgroundColor
 */
void monitorfiledlg::changeBGColor() {
    QColor f = QColorDialog::getColor(pbBG->paletteBackgroundColor());
    if (f.isValid()) {
	pbBG->setPaletteBackgroundColor(f);
	changed = true;
	toggleApplybtn();
    }
}

/*!
 * Like changeBGColor() but for foreground
 */
void monitorfiledlg::changeFGColor() {
    QColor f = QColorDialog::getColor(pbFG->paletteBackgroundColor());
    if (f.isValid()) {
	pbFG->setPaletteBackgroundColor(f);
	changed = true;
	toggleApplybtn();
    }
}

/*!
 * Like changeBGColor() but for graph color
 */
void monitorfiledlg::changeGraphColor() {
    QColor f = QColorDialog::getColor(pbGraph->paletteBackgroundColor());
    if (f.isValid()) {
	pbGraph->setPaletteBackgroundColor(f);
	changed = true;
	toggleApplybtn();
    }
}

void monitorfiledlg::exitDialog() {
    if (filechanged) {
	int i = QMessageBox::warning(this, 
		tr("Warning"),
		tr("File has changed\nWould you like to save?"),
		QMessageBox::Yes,
		QMessageBox::No,
		QMessageBox::Cancel
		);
	if (i == QMessageBox::Yes) {
	    saveFile();
	    accept();
	} else if (i == QMessageBox::No)
	    accept();
    } else 
	accept();
}

/*!
 * Opens a monitorpreviewdlg in which you can set the size and positon of a monitoring window
 */
void monitorfiledlg::showPreviewWindow() {
    monitorpreviewdlg *dlg;
    if (cbTitlebar->isChecked() == true) 
	dlg = new monitorpreviewdlg();
    else 
	dlg = new monitorpreviewdlg(0,0,true,Qt::WStyle_Customize|Qt::WStyle_NoBorder);

    if (sbX->value() >= 0 && sbY->value() >= 0) dlg->move(QPoint(sbX->value(), sbY->value()));
    if (sbW->value() > 0 && sbH->value() > 0) dlg->resize(sbW->value(), sbH->value());
    if (dlg->exec() == QDialog::Accepted) {
	if (dlg->cbPosition->isChecked()) {
	    sbX->setValue(dlg->curPos.x());
	    sbY->setValue(dlg->curPos.y());
	} else {
	    sbX->setValue(-1);
	    sbY->setValue(-1);
	}

	if (dlg->cbSize->isChecked()) {
	    sbW->setValue(dlg->curSize.width());
	    sbH->setValue(dlg->curSize.height());
	} else {
	    sbW->setValue(-1);
	    sbH->setValue(-1);
	}
    }
}

/*!
 * Opens a QFontDialog and sets the font of the current selected
 */
void monitorfiledlg::setFont()
{
    QFont f = QFontDialog::getFont(0, font);
    leFont->setText(f.family()+" "+QString::number(f.pointSize()));
    font = f;
}

void monitorfiledlg::checkDockmode() {
    bool enable = !cbDockmode->isChecked();
    sbX->setEnabled(enable);
    sbY->setEnabled(enable);
    tlx->setEnabled(enable);
    tly->setEnabled(enable);
    cbTitlebar->setEnabled(enable);
}
