/*
 * Copyright (C) Christof Musik, bitbone AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <qdir.h>
#include <qobject.h>
#include <qregexp.h>
#include <qfile.h>

#include "oidmap.h"

/*!
 * Create a instance of OIDMap and reads the default map. The default file is $HOME/.snmpmon/oid.map
 */
OIDMap::OIDMap() {
    mapfilename = QDir::homeDirPath()+"/.snmpmon/oid.map";
    openFile();
}

/*!
 * Creates a instance of OIDMap and reads the given filename.
 * \param name filename
 */
OIDMap::OIDMap(QString name) {
    mapfilename = name;
    openFile();
}

OIDMap::~OIDMap() {
}

/*!
 * Translate a key into an OID
 * \param key keyname
 * \return oid
 */
QString OIDMap::getOid (QString key) {
    if (mapoid.contains(key)) {
	return mapoid[key];
    } else {
	return QString::null;
    }
}

/*!
 * Translate a key into its unit
 * \param key keyname
 * \return unit
 */
int OIDMap::getUnit(QString key) {
    if (mapunit.contains(key)) { 
	return mapunit[key];
    } else {
	return 0;
    }
}

/*!
 * Checks if name exists in this map
 * \param name
 * \return true if name exists
 */
bool OIDMap::nameExists(QString name) {
    return mapoid.contains(name);
}

/*!
 * Opens mapfile and reads it content.
 * \see setMapFilename
 */
void OIDMap::openFile() {
    QFile *file = new QFile(mapfilename);
    if (file->open(IO_ReadOnly)) {
	QTextStream stream(file);
	QRegExp exp ("[\\s]*\\[(.*)\\][\\s]*\\{(.*)\\}[\\s]*(\\((.*)\\)){0,1}[\\s]*");
	QRegExp comment ("^[\\s]*#.*$");
	QRegExp newline ("^[\\s]*$");
	QString line;

	mapoid.clear();

	int ln = 1;
	while (!stream.atEnd()) {
	    line = stream.readLine();

	    if (comment.exactMatch(line));
	    else if (newline.exactMatch(line));
	    else if (exp.exactMatch(line)) {
		QString key = exp.cap(1);
		QString oid = exp.cap(2);
		bool ok;
		int unit = exp.cap(4).toInt(&ok, 10);
		if (!ok) unit = 0;

		mapoid[key] = oid;
		mapunit[key] = unit;
	    } else {
		std::cerr << QObject::tr("Error in file %1 on line %2.").arg(mapfilename).arg(ln) << std::endl;
	    }
	    ln++;
	}
	file->close();
    } else {
	std::cerr << QObject::tr("Couldn't open mapfile") << std::endl;
    }

    delete file;
}

/*!
 * Opens a OIDMap file and reads it content
 * \param f filename
 */
void OIDMap::openFile(QString f) {
    setMapFile(f);
    openFile();
}

/*!
 * \return list of all keys
 */
QStringList OIDMap::getKeys(void) {
    QMap<QString,QString>::Iterator it;
    QStringList list;
    for (it = mapoid.begin(); it != mapoid.end(); it++) {
	list.append(it.key());
    }
    return list;
}

/*!
 * \return QMap with all OIDs
 */
QMap<QString,QString> OIDMap::getMapOID() {
    return mapoid;
}


bool OIDMap::add(QString key, QString data) {
    if (mapoid.contains(key)) {
	return false;
    } else {
	mapoid[key] = data;
	return true;
    }
}

void OIDMap::erase(QString key) {
    mapoid.remove(key);
    mapunit.remove(key);
}

/*!
 * Sets a new oidmap in this instance. This is the map with the translated OIDs.
 */
void OIDMap::setMap(QMap<QString,QString> newmap) {
    mapoid = newmap;
}

/*!
 * Sets a new unitmap in this instance. This is the map with units.
 */
void OIDMap::setMap(QMap<QString,int> newmap) {
    mapunit = newmap;
}

/*!
 * Saves all entrys in this OIDMap to a file
 */
void OIDMap::save() {
    QFile file(mapfilename);
    if (file.open(IO_WriteOnly)) {
	QTextStream stream(&file);
	QMap<QString,QString>::Iterator it;

	for (it = mapoid.begin(); it != mapoid.end(); it++) {
	    stream << "[" << it.key() << "] {" << it.data() << "} (" << mapunit[it.key()] << ")" << endl;
	}

	file.close();
    } else {
	std::cerr << QObject::tr("Couldn't write to file %1.").arg(mapfilename) << std::endl;
    }
}

/*!
 * Sets a new default filename without the path. The default directory $HOME/.snmpmon/ will be used.
 * \param name name
 */
void OIDMap::setMapFilename(QString name) {
    mapfilename = QDir::homeDirPath()+"/.snmpmon/"+name;
}

/*!
 * Sets a new default filename with the path.
 * \param name name
 */
void OIDMap::setMapFile(QString name) {
    mapfilename = name;
}

/*!
 * \return name of the current mapfile
 */
QString OIDMap::getMapFilename() {
    return mapfilename;
}

/*!
 * \return map with all units
 */
QMap<QString,int> OIDMap::getMapUnit() {
    return mapunit;
}
