/*
 * Copyright (C) Christof Musik, bitbone AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <qapplication.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qregexp.h>
#include <qstatusbar.h>
#include <qtimer.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

#include "connectdlg.h"
#include "helpdialog.h"
#include "monitorfiledlg.h"
#include "oidmapeditor.h"
#include "prefdialog.h"
#include "snmpmonconfig.h"
#include "config.h"

QPtrList<SNMPMonitor> SNMPMonitor::monitor_list;

void SNMPMonitor::init()
{
    oidmap = new OIDMap();
    timer = new QTimer();
    connect( timer, SIGNAL( timeout() ), this, SLOT( putSNMPValue() ) );
    connect( Toolbar, SIGNAL( visibilityChanged(bool) ), this, SLOT( showtoolbar(bool) ) );

    oidinput->insertStringList(oidmap->getKeys());
    autostart = AUTOSTART;
    monitor_list.append(this);
    withdrawn = false;

    readSettings();
    setBackgroundColor( graph->getBackgroundColor() );
}

void SNMPMonitor::destroy()
{
    monitor_list.remove(this);
    if ( timer->isActive() ) stopTimer();
    if ( talker != NULL ) {
	talker->disconnect();
	delete talker;
    }
}

/*!
 * reads the current oidname from oidinput, translate the name into an oid and read the value from server
 * then it calls Graph::put(QString)
 */
void SNMPMonitor::putSNMPValue()
{
    string str;
    QString text = oidinput->currentText();
    QString oid_string = oidmap->getOid(text);
    graph->setUnitType( oidmap->getUnit(text) );

    if (oid_string != QString::null) {
	QStringList oid_list = QStringList::split("+", oid_string);
	QString put_string;
	for ( QStringList::Iterator it = oid_list.begin(); it != oid_list.end(); ++it ) {
	    QString stroid = *it;
	    if (talker->get( strdup(stroid.latin1()), &str ) == -1) {
		stopTimer();
		ac_startTimer->setEnabled(false);
		return;
	    }
	    put_string += QString( str.c_str() ) + "+";
	}

	// now send the put_string to the graph
	if ( !graph->put(put_string, text) ) {
	    graph->put(tr("Types doesn't match."));
	    stopTimer();
	}
    } else {
	graph->put(tr("No such name in map!"));
	stopTimer();
    }
}

/*!
 * Start the timer. This method is enabled by the start toolbar button.
 */
void SNMPMonitor::startTimer()
{
    int tout = timeout->value();
    putSNMPValue();
    ac_startTimer->setEnabled( false );
    ac_stopTimer->setEnabled( true );
    timer->start( tout*1000 );
    graph->setTimeout( tout );
    graph->cleanUp();
}

/*!
 * Stops the timer. This method is enabled by the stop toolbar button.
 */
void SNMPMonitor::stopTimer()
{
    ac_startTimer->setEnabled( true );
    ac_stopTimer->setEnabled( false );
    timer->stop();
}

/*!
 * Overloaded function. This calls SNMPMonitor::connectto(false).
 */
void SNMPMonitor::connectto()
{
    connectto(false);
}

/*!
 * shows the "Connect To Dialog"
 * \param exitoncancel if true then it will exit the application on cancel
 */
void SNMPMonitor::connectto( bool exitoncancel )
{
    if (talker == NULL) talker = new SNMPTalker();
    connectdlg *dlg = new connectdlg();
    dlg->server->insertStringList( knownserver );
    dlg->rereadsetting->setChecked( false );
    if (exitoncancel) {
	dlg->cancelbtn->setText( tr("Exit") );
	dlg->rereadsetting->hide();
	dlg->rereadsetting->setChecked( true );
    }
    SNMPTalker *nt = new SNMPTalker();
    do {
	if ( dlg->exec() == QDialog::Accepted ) {
	    char *srv = strdup( dlg->server->currentText().latin1() );
	    char *com = strdup( dlg->community->text().latin1() );
	    int ver = dlg->protocolversion->selectedId();
	    nt->reconnect( srv, com, ver );
	    if ( nt->isConnected() ) {
		setServer( QString(srv), QString(com), ver, autostart, dlg->rereadsetting->isChecked() );
	    } else {
		QMessageBox::warning(this, tr("No Connection"), tr("Couldn't connect to host %1").arg( srv ), QMessageBox::Ok, QMessageBox::NoButton);
		setCaption( tr("SNMP Monitor") );
		ac_startTimer->setEnabled(false);
		ac_stopTimer->setEnabled(false);
	    }
	} else {
	    if ( exitoncancel )
		exit(0);
	    else
		return;
	}
    } while ( !nt->isConnected() );
    if ( talker != NULL && talker->isConnected() && autostart )
	startTimer();
    delete dlg;
    return;
}

void SNMPMonitor::showCurrentValue( int i )
{
    if (statusBar()->isHidden()) 
	graph->showInlineValue();
    else
	statusBar()->message( tr("Current Value: ")+QString::number(i), 2000 );
}

/*!
 * Sets the timeout of this monitor.
 * \param t time in seconds
 */
void SNMPMonitor::setTimeout( int t )
{
    bool tb = timer->isActive();
    timer->changeInterval(t*1000);
    graph->setTimeout(t);
    if (!tb) timer->stop();
}

/*!
 * Shows the OIDMapEditor and sets the changed variables.
 */
void SNMPMonitor::show_oidmapeditor()
{
    oidmapeditor *dlg = new oidmapeditor();

    dlg->initMap(oidmap->getMapFilename(), cur_server);
    if ( dlg->exec() == QDialog::Accepted ) {
	oidmap->setMap( dlg->getOIDItems() );
	oidmap->setMap( dlg->getUnitItems() );
	if (dlg->getHost() != "")
	    oidmap->setMapFilename( "oid.map."+dlg->getHost() );
	else
	    oidmap->setMapFilename( "oid.map" );
	oidmap->save();
	oidinput->clear();
	oidinput->insertStringList( oidmap->getKeys() );
    }
    delete dlg;
}

/*!
 * Shows the settings dialog and sets the changed variables.
 */
void SNMPMonitor::show_settings()
{
    prefdialog *dlg = new prefdialog();
    dlg->initColors(graph->getBackgroundColor(), graph->getForegroundColor(), graph->getGraphColor());
    dlg->initHosts( knownserver );
    dlg->cbShowlines->setChecked( graph->getShowLines() );
    dlg->cbShowTline->setChecked( graph->getShowTimeline() );
    dlg->cbShowTlabel->setChecked( graph->getShowToplabel() );
    dlg->initFont( graph->getGraphFont() );
    dlg->graphTypeGroup->setButton( graph->getType() );
    if ( dlg->exec() == QDialog::Accepted ) {
	graph->setBackgroundColor( dlg->bg );
	graph->setForegroundColor( dlg->fg );
	graph->setGraphColor( dlg->graph );
	graph->setShowLines( dlg->cbShowlines->isChecked() );
	graph->setShowTimeline( dlg->cbShowTline->isChecked() );
	graph->setShowToplabel( dlg->cbShowTlabel->isChecked() );
	graph->setGraphFont( dlg->font );
	graph->setType( dlg->graphTypeGroup->selectedId() );
	knownserver = dlg->getKnownHosts();
	setPaletteBackgroundColor( graph->getBackgroundColor() );
    }
    delete dlg;
}

/*!
 * Shows the about dialog.
 */
void SNMPMonitor::show_about()
{
    QMessageBox::about(this, "SNMP Monitor " + QString::number(SNMPMON_VERSION),
	    tr( "SNMP Monitor is a small utility for monitoring SNMP devices\n\n"
		"copyright by Christof Musik, bitbone AG 2003-2004"));
}

/*!
 * Shows the help dialog.
 */
void SNMPMonitor::show_help()
{
    helpdialog *dlg = new helpdialog();
    dlg->show();
}

/*!
 * Saves the setting of this monitor\n
 * Default Destination: $HOME/.snmpmon/snmpmonrc
 * \see readSettings()
 */
void SNMPMonitor::saveSettings()
{
    QString filename = QDir::homeDirPath()+"/.snmpmon/snmpmonrc";
    QFile *file = new QFile(filename);
    if (file->open(IO_WriteOnly|IO_Truncate)) {
	QTextStream stream (file);
	stream << "[backgroundColor] {" << graph->getBackgroundColor().name() << "}" << endl;
	stream << "[foregroundColor] {" << graph->getForegroundColor().name() << "}" << endl;
	stream << "[graphColor] {" << graph->getGraphColor().name() << "}" << endl;
	stream << "[oidname] {" << oidinput->currentText() << "}" << endl;
	stream << "[showToolbar] {" << (ac_showtoolbar->isOn()? "true" : "false") << "}" << endl;
	stream << "[showMenubar] {" << (ac_showmenubar->isOn()? "true" : "false") << "}" << endl;
	stream << "[showStatusbar] {" << (ac_showstatusbar->isOn()? "true" : "false") << "}" << endl;
	stream << "[showTimeline] {" << (graph->getShowTimeline() ? "true" : "false") << "}" << endl;
	stream << "[showLabel] {" << (graph->getShowToplabel() ? "true" : "false") << "}" << endl;
	stream << "[knownServer] {" << knownserver.join(",") << "}" << endl;
	stream << "[size] {" <<  size().width() << "," << size().height() << "}" << endl;
	stream << "[showLines] {" << (graph->getShowLines() ? "true" : "false") << "}" << endl;
	stream << "[autostart] {" << (!ac_startTimer->isOn() ? "true" : "false") << "}" << endl;
	stream << "[font] {" << graph->getGraphFont().toString() << "}" << endl;
	stream << "[type] {" << graph->getType() << "}" << endl;
	file->close();
    } else {
	std::cerr << tr("Couldn't write to file %1.").arg(filename) << std::endl;
    }
    delete file;
}

/*!
 * Reads settings from the default destination.
 * \see saveSettings()
 */
void SNMPMonitor::readSettings()
{
    QString filename = QDir::homeDirPath()+"/.snmpmon/snmpmonrc";
    if ( QFileInfo(filename+"."+cur_server).isReadable() ) filename += "."+cur_server;
    QFile *file = new QFile( filename );
    if ( file->open(IO_ReadOnly) ) {
	QTextStream stream(file);
	QRegExp setting ("\\s*\\[\\s*(.*)\\s*\\].*\\{\\s*(.*)\\s*\\}.*");
	QRegExp comment ("^#.*$");
	QRegExp newline ("^[\\s]*$");
	QString line;
	int ln = 1;
	
	while( !stream.atEnd() ) {
	    line = stream.readLine();
	    if ( comment.exactMatch(line) );
	    else if ( newline.exactMatch(line) );
	    else if ( setting.exactMatch(line) ) {
		if ( setting.cap(1).lower() == "backgroundcolor" ) {
		    QRegExp colorsetting("[#0-9a-fA-F]");
		    if ( !colorsetting.exactMatch(setting.cap(2)) ) {
			QColor c(setting.cap(2));
			graph->setBackgroundColor(c);
			setBackgroundColor(c);
		    } else {
			std::cerr << tr("%1: Error in backgroundColor setting!").arg(ln) << std::endl;
		    }
		}
		else if ( setting.cap(1).lower() == "foregroundcolor" ) {
		    QRegExp colorsetting( "[#0-9a-fA-F]" );
		    if (!colorsetting.exactMatch( setting.cap(2) )) {
			QColor c( setting.cap(2) );
			graph->setForegroundColor(c);
		    } else {
			std::cerr << tr("%1: Error in foregroundColor setting!").arg(ln) << std::endl;
		    }
		}
		else if ( setting.cap(1).lower() == "graphcolor" ) {
		    QRegExp colorsetting("[#0-9a-fA-F]");
		    if ( !colorsetting.exactMatch(setting.cap(2)) ) {
			QColor c ( setting.cap(2) );
			graph->setGraphColor(c);
		    } else {
			std::cerr << tr("%1: Error in graphColor setting!").arg(ln) << std::endl;
		    }
		}
		else if ( setting.cap(1).lower() == "oidname" ) {
		    if ( oidmap->nameExists(setting.cap(2)) ) 
			oidinput->setCurrentText( setting.cap(2) );
		    else
			std::cerr << tr("%1: No such name in map!").arg(ln) << std::endl;
		}
		else if ( setting.cap(1).lower() == "showtoolbar" ) {
		    ac_showtoolbar->setOn( (setting.cap(2).lower() == "false" ? false : true) );
		}
		else if ( setting.cap(1).lower() == "showstatusbar" ) {
		    ac_showstatusbar->setOn( (setting.cap(2).lower() == "false" ? false : true) );
		}
		else if ( setting.cap(1).lower() == "showmenubar" ) {
		    ac_showmenubar->setOn( (setting.cap(2).lower() == "false" ? false : true) );
		}
		else if ( setting.cap(1).lower() == "knownserver" ) {
		    knownserver = QStringList::split(",", setting.cap(2));
		}
		else if ( setting.cap(1).lower() == "size" ) {
		    QRegExp size ("\\s*(\\d*)\\s*,\\s*(\\d*)\\s*");
		    if ( size.exactMatch(setting.cap(2)) ) {
			int sizex = size.cap(1).toInt();
			int sizey = size.cap(2).toInt();

			if ( sizex > 0 && sizey > 0 ) {
			    resize( sizex, sizey );
			}
		    } else {
			std::cerr << tr("%1: Error in size setting!").arg(ln) << std::endl;
		    }
		}
		else if ( setting.cap(1).lower() == "autostart" ) {
		    autostart = setting.cap(2).lower() == "true" ? true : false;
		}
		else if ( setting.cap(1).lower() == "showlines" ) {
		    graph->setShowLines( setting.cap(2).lower() == "false" ? false : true );
		}
		else if ( setting.cap(1).lower() == "font" ) {
		    QFont f;
		    if ( f.fromString(setting.cap(2)) )
			graph->setGraphFont(f);
		}
		else if ( setting.cap(1).lower() == "type" ) {
		    graph->setType( setting.cap(2).toInt() );
		}
		else if ( setting.cap(1).lower() == "showtimeline" ) {
		    graph->setShowTimeline( setting.cap(2).lower() == "true" ? true : false );
		}
		else if ( setting.cap(1).lower() == "showlabel" ) {
		    graph->setShowToplabel(setting.cap(2).lower() == "true"  ? true : false );
		}
		else {
		    std::cerr << tr("Error in file %1 on line %2.").arg(filename).arg(ln) << std::endl;
		}
	    } else {
		std::cerr << tr("Error in file %1 on line %2.").arg(filename).arg(ln) << std::endl;
	    }
	    ln++;
	}
    } else {
	std::cerr << tr("Couldn't open file %1").arg(filename) << std::endl;
    }
    delete file;
}

/*!
 * Show or hides the toolbar
 * \param s if true toolbar is shown
 */
void SNMPMonitor::showtoolbar( bool s )
{
    s ? Toolbar->show() : Toolbar->hide();
    ac_showtoolbar->setOn(s);
}

/*!
 * Show or hides the statusbar 
 * \param s if true statusbar is shown
 */
void SNMPMonitor::showstatusbar( bool s )
{
    s ? statusBar()->show() : statusBar()->hide();
    ac_showstatusbar->setOn(s);
}

/*!
 * Show or hides the menubar 
 * \param s if true menubar is shown
 */
void SNMPMonitor::showmenubar( bool s )
{
    s ? MenuBar->show() : MenuBar->hide();
    ac_showmenubar->setOn(s);
}

/*!
 * First opens a filedialog asking for a Monitor file, than calls readMonitorFile(QString) with the selected file
 */
void SNMPMonitor::openNewGraphs()
{
    QString filename = QFileDialog::getOpenFileName( QDir::currentDirPath(), tr("Monitor Files (*.mon);; All Files (*)") );
    readMonitorFile(filename);
}

/*!
 * Creates a new SNMPMonitor and sets the default values for this new one
 */
void SNMPMonitor::fileNew()
{
    QString srv, com;
    int ver;
    
    connectdlg *dlg = new connectdlg();
    dlg->server->insertStringList(knownserver);
    do {
	if (dlg->exec() == QDialog::Accepted) {
	    srv = dlg->server->currentText();
	    com = dlg->community->text();
	    ver = dlg->protocolversion->selectedId();
	    if (SNMPTalker::checkServer(strdup(srv.latin1()), strdup(com.latin1()), ver)) {
		SNMPMonitor *a = new SNMPMonitor(0,0,Qt::WDestructiveClose);
		a->setServer(srv, com, ver, autostart, dlg->rereadsetting->isChecked());
		a->show();
	    }
	} else 
	    return;
    }
    while ( !SNMPTalker::checkServer(strdup(srv.latin1()), strdup(com.latin1()), ver) );
}

void SNMPMonitor::fileExit()
{
    exit(0);
}

void SNMPMonitor::fileClose()
{
    close();
}

/*!
 * sets a new server for this monitor
 * \param s servername
 * \param c communityname
 * \param v version of the protocol
 * \param a if true autostart is enabled
 * \param rereadsetting if true settings will be set to default
 */
void SNMPMonitor::setServer( QString s, QString c, int v, bool a, bool rereadsetting )
{
    char *srv = strdup(s.latin1());
    char *com = strdup(c.latin1());
    
    talker = new SNMPTalker(srv, com, v);
    
    cur_server = srv;
    cur_community = com;
    cur_version = v;

    QString oidfile = QDir::homeDirPath()+"/.snmpmon/oid.map."+cur_server;
    QString ctext = oidinput->currentText();
    if (QFileInfo(oidfile).isReadable()) {
	oidmap->openFile(oidfile);
	oidinput->clear();
	oidinput->insertStringList( oidmap->getKeys() );
    } else {
	oidmap->openFile(QDir::homeDirPath()+"/.snmpmon/oid.map");
	oidinput->clear();
	oidinput->insertStringList( oidmap->getKeys() );
    }
    if (oidmap->nameExists(ctext)) oidinput->setCurrentText(ctext);
    if (rereadsetting)
	readSettings();
    graph->cleanUp();
    graph->setConnectedServer(srv);
    setCaption("SNMP Monitor - "+QString::fromAscii(srv));
    if (a)
	startTimer();
    else {
	ac_startTimer->setEnabled(true);
	ac_stopTimer->setEnabled(false);
	stopTimer();
    }
    free(srv);
    free(com);
}

/*!
 * Reads the given monitorfile and creates new instances of SNMPMonitor with the given configuration values
 * \param f filename
 */
void SNMPMonitor::readMonitorFile( QString f )
{
    if (f == QString::null || f == "") {
	return;
    }

    QFile file(f);
    if ( file.open(IO_ReadOnly) ) {
	QTextStream stream(&file);
	QString line;
	QRegExp configend ("^\\s*\\[end\\].*");
	SNMPMonConfig *cfg = new SNMPMonConfig();

	int ln = 1;
	while( !stream.atEnd() ) {
	    line = stream.readLine();
	    if ( configend.exactMatch(line) ) {
		if ( cfg->server != "" && cfg->oidname != "" ) {
		    SNMPMonitor *a = new SNMPMonitor();
		    a->setConfig( cfg );
		} else {
		    std::cerr << tr( "Server or oidname not defined" ) << std::endl;
		}
		delete cfg;
		cfg = new SNMPMonConfig();	
	    } else {
		if ( !cfg->addConfigString(line) ) {
		    std::cerr << tr( "Error on line %1" ).arg( ln ) << std::endl;
		}
	    }
	    ln++;
	}
	delete cfg;
    } else {
	std::cerr << tr( "Couldn't open file %1" ).arg( f ) << std::endl;
    }

    SNMPMonitor *mon = monitor_list.at(1);
    for ( ; mon; mon = monitor_list.next()) {
	if ( !mon->isVisible() ) mon->show();
    }
    delete mon;
}

/*!
 * Gets the current config-string and appends it to a file
 */
void SNMPMonitor::addToMonitorFile()
{
    QString filename = QFileDialog::getSaveFileName( QDir::currentDirPath(), tr("Monitor Files (*.mon);; All Files (*)") );
    if (filename != QString::null && filename != "") {
	QFile file( filename );
	if ( file.open(IO_WriteOnly | IO_Append) ) {
	    QTextStream stream( &file );
	    stream << getCurrentConfigString();
	} else {
	    QMessageBox::warning(this, tr("Permission denied"), tr( "Couldn't write to file %1" ).arg( filename ), QMessageBox::Ok, 0 );
	    std::cerr << tr( "Couldn't write to file %1" ).arg( filename ) << std::endl;
	}
	file.close();
    }
}

/*!
 * Show or hide the window border. This includes the titlebar.\n
 * Note: on some windowmanagers this won't work
 * \param s if true window border will be shown
 */
void SNMPMonitor::showborder( bool s )
{
    WFlags flags = getWFlags();
    if (!s) {
	flags |= Qt::WStyle_Customize;
	flags |= Qt::WStyle_NoBorder;
	
	QPoint p( geometry().x(), geometry().y() );
	reparent(0, flags, p, true);
    } else {
	flags ^= Qt::WStyle_NoBorder;
	flags ^= Qt::WStyle_Customize;

	reparent( 0, flags, pos(), true );
    }
    hideFromTaskbar( winId(), !s );
}

/*
 * This method opens a context menu of the current monitor
 */
void SNMPMonitor::contextMenuEvent( QContextMenuEvent * )
{
    QPopupMenu* contextMenu = new QPopupMenu( this );
    QPopupMenu* viewMenu = new QPopupMenu( this );
    QPopupMenu* editMenu = new QPopupMenu( this );
    QPopupMenu* oidMenu = new QPopupMenu( this );

    connect ( oidMenu, SIGNAL(activated(int)), this, SLOT(setCurrentOID(int)) );
    
    QStringList oidkeys = oidmap->getKeys();
    int i = 0;
    for ( QStringList::Iterator it = oidkeys.begin(); it != oidkeys.end(); ++it ) {
	oidMenu->insertItem(*it, i);
	if ( *it == oidinput->currentText() ) oidMenu->setItemChecked(i, true);
	i++;
    }
    contextMenu->insertItem( tr("Select OID"), oidMenu );
    contextMenu->insertSeparator();

    ac_new->addTo( contextMenu );
    ac_connectto->addTo( contextMenu );
    ac_addtomfile->addTo( contextMenu );
    ac_saveallmonitors->addTo( contextMenu );
    contextMenu->insertSeparator();
    
    ac_startTimer->addTo( contextMenu );
    ac_stopTimer->addTo( contextMenu );
    ac_removemax->addTo( contextMenu );
    contextMenu->insertSeparator();

    // edit menu
    ac_oidmapeditor->addTo( editMenu );
    ac_monfileeditor->addTo( editMenu );
    ac_settings->addTo( editMenu );
    editMenu->insertSeparator();
    ac_savesettings->addTo( editMenu );
    
    // view menu
    ac_showmenubar->addTo( viewMenu );
    ac_showtoolbar->addTo( viewMenu );
    ac_showstatusbar->addTo( viewMenu );
    if ( !withdrawn ) ac_showborder->addTo( viewMenu );
    viewMenu->insertSeparator();
    ac_cleangraph->addTo( viewMenu );
   
    contextMenu->insertItem( tr("Edit"), editMenu );
    contextMenu->insertItem( tr("View"), viewMenu );
    contextMenu->insertSeparator();
    ac_close->addTo( contextMenu );
    ac_exit->addTo( contextMenu );
    contextMenu->exec( QCursor::pos() );
    delete oidMenu;
    delete viewMenu;
    delete editMenu;
    delete contextMenu;
}

void SNMPMonitor::setCurrentOID( int id ) {
    oidinput->setCurrentItem( id );
}

/*!
 * Shows the monitorfile editor
 */
void SNMPMonitor::show_monfileeditor()
{
    monitorfiledlg *dlg = new monitorfiledlg();
    dlg->cbOidname->insertStringList(oidmap->getKeys());
    dlg->exec();
    delete dlg;
}

void SNMPMonitor::mousePressEvent( QMouseEvent *e )
{
    if ( withdrawn ) return;
    clickPos = QPoint( e->pos().x() + ( geometry().x() - pos().x() ), e->pos().y() + ( geometry().y() - pos().y() ) );
    clickButton = e->button();
}

void SNMPMonitor::mouseMoveEvent( QMouseEvent *e )
{
    if ( withdrawn ) return;
    if ( clickButton == Qt::LeftButton ) {
	move( e->globalPos() - clickPos );
    }
}

/*!
 * Sets all config from cfg to this current monitor
 * \param cfg config
 * \see SNMPMonConfig
 */
void SNMPMonitor::setConfig( SNMPMonConfig *cfg )
{
    setServer( cfg->server, cfg->community, cfg->version, cfg->autostart );
    int flags = Qt::WDestructiveClose;
    if (cfg->withdrawn) setWithdrawnState();
    else {
	if (cfg->titlebar == false) {
	    flags |= Qt::WStyle_Customize|Qt::WStyle_NoBorder;
	    ac_showborder->setOn(false);
	} else {
	    ac_showborder->setOn(true);
	}
    }

    timeout->setValue( cfg->timeout );
    cur_oid = cfg->oidname;
    graph->setGraphColor( cfg->graph );
    graph->setBackgroundColor( cfg->bg );
    setBackgroundColor( cfg->bg );
    graph->setForegroundColor( cfg->fg );
    graph->setNamedOID( cfg->oidname );
    graph->setShowLines( cfg->lines );
    graph->setShowTimeline( cfg->timeline );
    oidinput->setCurrentText( cfg->oidname );
    showtoolbar( cfg->toolbar );
    showmenubar( cfg->menubar );
    showstatusbar( cfg->statusbar );
    if ( cfg->w > 0 && cfg->h > 0 ) {
	resize( QSize(cfg->w, cfg->h) );
    }
    if ( cfg->x >= 0 && cfg->y >= 0 ) {
	move( QPoint(cfg->x,cfg->y) );
    }
    
    setWFlags( flags );
}

/*!
 * Popups the oidinput-menu from the toolbar
 */
void SNMPMonitor::popupOIDName()
{
    oidinput->popup();
}

/*!
 * Returns a whole config string of the current monitor
 */
QString SNMPMonitor::getCurrentConfigString()
{
    QString str;
    str  = "[server] {" + cur_server + "}\n";
    str += "[oidname] {" + oidinput->currentText() + "}\n";
    str += "[version] {" + QString::number(cur_version) + "}\n";
    str += "[community] {" + cur_community + "}\n";
    str += "[timeout] {" + QString::number(timeout->value()) + "}\n";
    str += "[backgroundColor] {" + graph->getBackgroundColor().name() + "}\n";
    str += "[foregroundColor] {" + graph->getForegroundColor().name() + "}\n";
    str += "[graphColor] {" + graph->getGraphColor().name() + "}\n";
    str += "[autoStart] {";
    timer->isActive() ?str += "true" : str+= "false";
    str += "}\n";
    str += "[showToolbar] {";
    ac_showtoolbar->isOn() ? str +=  "true" : str+= "false";
    str += "}\n";
    str += "[showMenubar] {";
    ac_showmenubar->isOn() ? str +=  "true" : str+= "false";
    str += "}\n";
    str += "[showStatusbar] {";
    ac_showstatusbar->isOn() ? str +=  "true" : str+= "false";
    str += "}\n";
    str += "[showTitlebar] {";
    ac_showborder->isOn() ? str += "true" : str += "false";
    str += "}\n";
    str += "[showLines] {";
    graph->getShowLines() ? str += "true" : str += "false";
    str += "}\n";
    str += "[showTimeline] {";
    graph->getShowTimeline() ? str += "true" : str += "false";
    str += "}\n";
    str += "[withDrawn] {";
    withdrawn ? str += "true" : str += "false";
    str += "}\n";
    str += "[font] {" + graph->getGraphFont().toString() + "}\n";
    str += "[geometry] {" + QString::number(pos().x()) + "," + QString::number(pos().y()) + "," + QString::number(size().width()) + "," + QString::number(size().height()) + "}\n";
    str += "[end]\n\n";
    return str;
}

/*!
 * Saves all visible monitors in a file.
 */
void SNMPMonitor::saveAllMonitors()
{
    QString filename = QFileDialog::getSaveFileName( QDir::currentDirPath(), tr("Monitor Files (*.mon);; All Files (*)") );
    if (filename != "") {
	QFile file(filename);
	if (file.open(IO_WriteOnly)) {
	    QTextStream stream(&file);

	    SNMPMonitor *mon;
	    for (mon = monitor_list.first(); mon; mon = monitor_list.next()) {
		if (mon->isVisible()) stream << mon->getCurrentConfigString();
	    }
	    delete mon;
	} else {
	    std::cerr << tr("Couldn't write to file %1").arg(filename) << std::endl;
	}
	file.close();
    }
}

/*!
 * WORKAROUND: Sets the background color of the whole widget, to get rid of the line at the top and bottom for a window without titlebar
 */
void SNMPMonitor::setBackgroundColor( QColor c )
{
    QColor bgcolor = QApplication::palette(this).active().background();
    setPaletteBackgroundColor(c);

    MenuBar->setBackgroundMode(Qt::PaletteBackground);
    MenuBar->setPaletteBackgroundColor(bgcolor);
    statusBar()->setBackgroundMode(Qt::PaletteBackground);
    statusBar()->setPaletteBackgroundColor(bgcolor);
    Toolbar->setBackgroundMode(Qt::PaletteBackground);
    Toolbar->setPaletteBackgroundColor(bgcolor);
}

void SNMPMonitor::hideFromTaskbar (WId winid, bool add) 
{
    sendXEvent(winid, "_NET_WM_STATE", "_NET_WM_STATE_SKIP_PAGER", "_NET_WM_STATE_SKIP_TASKBAR", add);
}

void SNMPMonitor::sendXEvent(WId winid, QString msg0, QString msg1, QString msg2, bool add) 
{
    XEvent event;
    Display *disp = XOpenDisplay( getenv("DISPLAY") );

    Atom type = XInternAtom(disp, msg0.latin1(), false);
    Atom data1 = XInternAtom(disp, msg1.latin1(), false);
    Atom data2 = XInternAtom(disp, msg2.latin1(), false);

    Window win = winid;
    event.type = ClientMessage;

    event.xclient.type = ClientMessage;
    event.xclient.send_event = true;
    event.xclient.message_type = type;
    event.xclient.window = win;
    event.xclient.format = 32;
    event.xclient.data.l[0] = add ? 1 : 0;
    event.xclient.data.l[1] = data1;
    event.xclient.data.l[2] = data2;
    event.xclient.data.l[3] = 0l;

    if ( !XSendEvent( disp, DefaultRootWindow(disp), false, (SubstructureRedirectMask|SubstructureNotifyMask), &event ) )
	std::cerr << "Error on sending XEvent!" << std::endl;

    XCloseDisplay(disp);
}

void SNMPMonitor::setWithdrawnState()
{
    Display *disp = XOpenDisplay( getenv("DISPLAY") );
    Window win = winId();

    XWMHints mywmhints; 
    mywmhints.initial_state = WithdrawnState; 
    mywmhints.flags = StateHint;

    XSetWMHints(disp, win, &mywmhints);

    withdrawn = true;

    XCloseDisplay(disp);
}

void SNMPMonitor::setWidth( int w )
{
    if (w > 0) resize(w,height());
}

void SNMPMonitor::setHeight( int h )
{
    if (h > 0) resize(width(),h);
}
