/*
 * Copyright (C) Christof Musik, bitbone AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <qstring.h>
#include <qregexp.h>
#include <qcolor.h>
#include <qpainter.h>

#include "snmpmonconfig.h"
#include "graph.h"
#include "config.h"

SNMPMonConfig::SNMPMonConfig() {
    setDefault();
}

SNMPMonConfig::~SNMPMonConfig() {

}

/*!
 * Sets default settings
 */
void SNMPMonConfig::setDefault() {
    server = "";
    community = "public";
    oidname = "";
    
    bg = QColor(BACKGROUND_COLOR);
    fg = QColor(FOREGROUND_COLOR);
    graph = QColor(GRAPH_COLOR);
    font = QFont();
   
    version = PROTOCOL_VERSION;
    timeout = TIMEOUT;
    x = X_POS;
    y = Y_POS;
    w = WIDTH;
    h = HEIGHT;
    type = GRAPHTYPE;
    toolbar = SHOW_TOOLBAR;
    menubar = SHOW_MENUBAR;
    statusbar = SHOW_STATUSBAR;
    titlebar = SHOW_TITLEBAR;
    autostart = AUTOSTART;
    lines = SHOW_LINES;
    timeline = SHOW_TIMELINE;
    withdrawn = WITHDRAWN_STATE;
    toplabel = SHOW_TOPLABEL;
}

/*!
 * \return String that can be written to a file
 */
QString SNMPMonConfig::getSaveString() {
    QString str;
    if (server != "" && oidname != "") {
	str = "[server] {" + server + "}\n";
	str += "[oidname] {" + oidname + "}\n";
	str += "[version] {" + QString::number(version) + "}\n";
	str += "[community] {" + community + "}\n";
	str += "[timeout] {" + QString::number(timeout) + "}\n";
	str += "[backgroundColor] {" + bg.name() + "}\n";
	str += "[foregroundColor] {" + fg.name() + "}\n";
	str += "[graphColor] {" + graph.name() + "}\n";
	str += "[autoStart] {";
	autostart ?str += "true" :str+= "false";
	str += "}\n";
	str += "[showToolbar] {";
	toolbar?str +=  "true" :str+= "false";
	str += "}\n";
	str += "[showMenubar] {";
	menubar?str +=  "true" :str+= "false";
	str += "}\n";
	str += "[showStatusbar] {";
	statusbar?str +=  "true" :str+= "false";
	str += "}\n";
	str += "[showTitlebar] {";
	titlebar?str += "true" :str+= "false";
	str += "}\n";
	str += "[showLines] {";
	lines ? str += "true" : str += "false";
	str += "}\n";
	str += "[showTimeline] {";
	timeline ? str += "true" : str += "false";
	str += "}\n";
	str += "[withDrawn] {";
	withdrawn ? str += "true" : str += "false";
	str += "}\n";
	str += "[showLabel] {";
	toplabel ? str += "true" : str += "false";
	str += "}\n";
	str += "[font] {" + font.key() + "}\n";
	str += "[type] {" + QString::number(type) + "}\n";
	if (w >= 0 && h >= 0) 
	    str += "[geometry] {" + QString::number(x)+ "," + QString::number(y)+ "," + QString::number(w) + "," + QString::number(h) + "}\n";
	str += "[end]\n";

	return str;
    } else {
	return "";
    }
}

/*!
 * Extract needed data from line and set the right setting\n
 * \return true on success (no error in setting) else returns false
 */
bool SNMPMonConfig::addConfigString(QString line) {
    QRegExp setting("^\\s*\\[\\s*(.*)\\s*\\].*\\{\\s*(.*)\\s*\\}.*");
    QRegExp comment ("^#.*$");
    QRegExp newline ("^[\\s]*$");

    if (comment.exactMatch(line));
    else if (newline.exactMatch(line));
    else if (setting.exactMatch(line)) {
	if (setting.cap(1).lower() == "server") {
	    server = setting.cap(2);
	}
	else if (setting.cap(1).lower() == "community") {
	    if (setting.cap(2) != "")
		community = setting.cap(2);
	}
	else if (setting.cap(1).lower() == "version") {
	    version = setting.cap(2).toInt();
	}
	else if (setting.cap(1).lower() == "oidname") {
	    oidname = setting.cap(2);
	}
	else if (setting.cap(1).lower() == "graphcolor") {
	    QRegExp colorsetting("[#0-9a-fA-F]");
	    if (!colorsetting.exactMatch(setting.cap(2))) {
		graph = QColor(setting.cap(2));
	    } else {
		return false;
	    }
	}
	else if (setting.cap(1).lower() == "backgroundcolor") {
	    QRegExp colorsetting("[#0-9a-fA-F]");
	    if (!colorsetting.exactMatch(setting.cap(2))) {
		bg = QColor(setting.cap(2));
	    } else {
		return false;
	    }
	}
	else if (setting.cap(1).lower() == "foregroundcolor") {
	    QRegExp colorsetting("[#0-9a-fA-F]");
	    if (!colorsetting.exactMatch(setting.cap(2))) {
		fg = QColor(setting.cap(2));
	    } else {
		return false;
	    }
	}
	else if (setting.cap(1).lower() == "showtoolbar") {
	    if (setting.cap(2) == "false")
		toolbar = false;
	}
	else if (setting.cap(1).lower() == "showmenubar") {
	    if (setting.cap(2) == "false")
		menubar = false;
	}
	else if (setting.cap(1).lower() == "showstatusbar") {
	    if (setting.cap(2) == "false")
		statusbar = false;
	}
	else if (setting.cap(1).lower() == "autostart") {
	    if (setting.cap(2) == "true")
		autostart = true;
	}
	else if (setting.cap(1).lower() == "timeout") {
	    timeout = setting.cap(2).toInt();
	    if (timeout > 60) timeout = 60;
	}
	else if (setting.cap(1).lower() == "geometry") {
	    QRegExp size ("\\s*([-]?\\d*)\\s*,\\s*([-]?\\d*)\\s*,\\s*([-]?\\d*)\\s*,\\s*([-]?\\d*)\\s*");
	    if (size.exactMatch(setting.cap(2))) {
		x = size.cap(1).toInt();
		y = size.cap(2).toInt();
		w = size.cap(3).toInt();
		h = size.cap(4).toInt();
	    }
	}
	else if (setting.cap(1).lower() == "showtitlebar") {
	    if (setting.cap(2) == "false") {
		titlebar = false;
	    }
	}
	else if (setting.cap(1).lower() == "font") {
	    QFont f;
	    if (f.fromString(setting.cap(2)))
		font = f;
	}
	else if (setting.cap(1).lower() == "showlines") {
	    if (setting.cap(2).lower() == "false")
		lines = false;
	    else
		lines = true;
	}
	else if (setting.cap(1).lower() == "type") {
	    type = setting.cap(2).toInt();
	}
	else if (setting.cap(1).lower() == "showtimeline") {
	    if (setting.cap(2) == "true")
		timeline = true;
	    else
		timeline = false;
	}
	else if (setting.cap(1).lower() == "withdrawn") {
	    if (setting.cap(2) == "true")
		withdrawn = true;
	}
	else if (setting.cap(1).lower() == "showlabel") {
	    if (setting.cap(2) == "true")
		toplabel = true;
	}
	else {
	    return false;
	}
    }
    else 
	return false;
    return true;
}
