/*
 * Copyright (C) Christof Musik, bitbone AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "snmptalker.h"

SNMPTalker::SNMPTalker(void) {
    status_synch = -1;
    status_read_objid = -1;
    cur_server = "";
    cur_community = "";
    cur_version = 0;
}

SNMPTalker::SNMPTalker(char *server, char *community, int version) {
    connect(server, community, version);
}

SNMPTalker::~SNMPTalker(void) {
    if (isConnected()) snmp_close(ss);
}

/*!
 * Connects to a given server
 * \param server
 * \param community
 * \param version
 */
void SNMPTalker::connect(char* server, char* community, int version) {
    init_snmp("snmpmon "+*server);
    snmp_sess_init(&session);
    session.peername = server;

    switch (version) {
	case 1:
	    session.version = SNMP_VERSION_1;
	    break;
	case 2:
	    session.version = SNMP_VERSION_2c;
	    break;
	case 3:
	    session.version = SNMP_VERSION_3;
	    break;
	default:
	    session.version = SNMP_VERSION_1;
    }

    session.community = (u_char*) community;
    session.community_len = strlen((char*) session.community);

    ss = snmp_open(&session);
    if (!ss) {
	std::cerr << "Couldn't connect to " << server << std::endl;
    } else {
	cur_server = std::string(server);
	cur_community = std::string(community);
	cur_version = version;
    }
}

void SNMPTalker::disconnect() {
    if (isConnected()) snmp_close(ss);
}

/*!
 * Reconnects to a given server
 * \param server
 * \param community
 * \param version
 */
void SNMPTalker::reconnect(char *server, char *community, int version) {
    disconnect();
    connect(server, community, version);
}

/*!
 * \return true if connected
 */
bool SNMPTalker::isConnected() {
    return ss ? true : false;
}

struct snmp_pdu* SNMPTalker::read(char* objectid) {
    if (isConnected()) {
	oid anOID[MAX_OID_LEN];
	size_t anOID_len = MAX_OID_LEN;

	struct snmp_pdu *pdu, *response;
	init_mib();	
	pdu = snmp_pdu_create(SNMP_MSG_GET);
	status_read_objid = read_objid(objectid, anOID, &anOID_len);

	snmp_add_null_var(pdu, anOID, anOID_len);
	status_synch = snmp_synch_response(ss, pdu, &response);
	return response;
    } else {
	return NULL;
    }
}

/*!
 * Reads from a SNMP-device.
 * \param objectid 
 * \param ret pointer to a string, in which the result will be written
 * \return on success returns number of bytes else return -1
 */
int SNMPTalker::get(char* objectid, string* ret) {

    struct snmp_pdu *response = read(objectid);
    if (!isConnected() || response == NULL) { 
	*ret = string(".1= STRING: ERROR no connection");
	return -1;
    }
    free(objectid);

    if (response->errstat == SNMP_ERR_NOERROR && status_synch == STAT_SUCCESS) {
	struct variable_list *vars;
	char val[1024];
	int count = 0;
	for(vars = response->variables; vars; vars = vars->next_variable) {
	    count = snprint_variable(val, sizeof(val), vars->name, vars->name_length, vars);
	}
	*ret = string((char *) val);

	delete vars;
	snmp_free_pdu(response);
	return count;
    } else {
	snmp_free_pdu(response);
	*ret = string(".1= STRING: ERROR while reading");
	return -1;
    }
}

/*!
 * Reads from a SNMP-device.
 * \param objectid
 * \return string from device
 */
string SNMPTalker::get(char* objectid) {
    if (!isConnected()) return ".1= STRING: ERROR no connection";

    struct snmp_pdu *response = read(objectid);
    free(objectid);
    if (response->errstat == SNMP_ERR_NOERROR && status_synch == STAT_SUCCESS) {
	struct variable_list *vars;
	char val[1024];

	for(vars = response->variables; vars; vars = vars->next_variable) {
	    snprint_variable(val, sizeof(val), vars->name, vars->name_length, vars);
	}

	delete vars;
	return string((char *)val);
    } else {
	snmp_free_pdu(response);
	return ".1= STRING: ERROR while reading";
    }
}

/*!
 * Checks if server is available
 * \param srv servername
 * \param com communityname
 * \param ver protocol version
 */
bool SNMPTalker::checkServer(char* srv, char* com, int ver) {
    struct snmp_session session;
    struct snmp_session *ss;
    init_snmp("snmpmon "+*srv);
    snmp_sess_init(&session);
    session.peername = srv;

    switch (ver) {
	case 1:
	    session.version = SNMP_VERSION_1;
	    break;
	case 2:
	    session.version = SNMP_VERSION_2c;
	    break;
	case 3:
	    session.version = SNMP_VERSION_3;
	    break;
	default:
	    session.version = SNMP_VERSION_1;
    }

    session.community = (u_char*) com;
    session.community_len = strlen((char*) session.community);

    ss = snmp_open(&session);
    return ss ? true : false;
}

/*!
 * Checks if the given oid is valid
 * \param oidname
 */
bool SNMPTalker::checkOID (char *oidname) {
    oid name[MAX_OID_LEN];
    size_t name_length = MAX_OID_LEN;
    return snmp_parse_oid( oidname, name, &name_length );
}
